searchNodes=[{"doc":"","ref":"dactyl.html","title":"dactyl","type":"module"},{"doc":"build a #dactyl_template{} from a source string or binary","ref":"dactyl.html#compile/1","title":"dactyl.compile/1","type":"function"},{"doc":"build a #dactyl_template{} from a source file","ref":"dactyl.html#compile_file/1","title":"dactyl.compile_file/1","type":"function"},{"doc":"Build an internal representation of a dactyl template","ref":"dactyl.html#new_template/1","title":"dactyl.new_template/1","type":"function"},{"doc":"render from a template","ref":"dactyl.html#render/2","title":"dactyl.render/2","type":"function"},{"doc":"render from a template using module callbacks","ref":"dactyl.html#render/3","title":"dactyl.render/3","type":"function"},{"doc":"compile a source file, build a template, then render it","ref":"dactyl.html#render_file/2","title":"dactyl.render_file/2","type":"function"},{"doc":"compile a source file, build a template, then render it with callbacks","ref":"dactyl.html#render_file/3","title":"dactyl.render_file/3","type":"function"},{"doc":"","ref":"dactyl.html#t:arg/0","title":"dactyl.arg/0","type":"type"},{"doc":"","ref":"dactyl.html#t:args/0","title":"dactyl.args/0","type":"type"},{"doc":"","ref":"dactyl.html#t:dactyl_template/0","title":"dactyl.dactyl_template/0","type":"opaque"},{"doc":"","ref":"dactyl.html#t:operation/0","title":"dactyl.operation/0","type":"type"},{"doc":"","ref":"dactyl.html#t:segment/0","title":"dactyl.segment/0","type":"type"},{"doc":"Dactyl is a simple string templating library for Erlang. It's not especially fast, but does have quite a bit of power. With Dactyl you can perform basic (named) parameter expansion, as well as conditional expansion, list expansion, and custom formatting.","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"Dactyl follows a very simple format for identifying named parameters to a template. The ~ character is used to prefix the name of a parameter. At the end of the name, a term operation specifies how the parameter is to be expanded. The parameter values are passed into the render/2 function as a property list. Other templating libraries for Erlang use dictionaries, but property lists can be quite fast for small lists (up to ~20) and are much easier to debug if there is a problem. Simple example: 1 &gt; dactyl : render ( &quot; ~n ame~;&quot; , [ { name , &quot;Jeff&quot; } ] ) . &quot;Jeff&quot; The above is the most basic example of using Dactyl. The render/2 function compiles the template string into a #dactyl_template{} record, which is then expanded using the parameter values: [{name,&quot;Jeff&quot;}] . You can also choose to pre-compile the template and then render it multiple times using different parameter value proplists: 2 &gt; { ok , Template } = dactyl : compile ( &quot; ~n ame~;&quot; ) . { ok , . . . } 3 &gt; dactyl : render ( Template , [ { name , &quot;Bob&quot; } ] ) . &quot;Bob&quot; Dactyl is also capable of compiling files as templates, which can then be used like any other template: 4 &gt; { ok , Home } = dactyl : compile_file ( &quot;home.html&quot; ) . { ok , . . . }","ref":"readme.html#the-basics","title":"Overview - The basics","type":"extras"},{"doc":"Dactyl uses a syntax for formatting parameters similar to that of Common Lisp's FORMAT function. It isn't necessary, however, to be a black-belt in FORMAT to use Dactyl, as it only uses a fraction of its features. Every named parameter begins with the ~ character and is followed by the name of the parameter. The value of this parameter is what will be expanded when the template is rendered. How the value is rendered depends on the operation that is supplied after the parameter name. Basic substitution The most common operation is the ~; function, or &quot;basic&quot; substitution. It simply means that whatever the value of the parameter, convert it to a string and render it. 5 &gt; dactyl : render ( &quot;I am ~age~; years old.&quot; , [ { age , 36 } ] ) . &quot;I am 36 years old.&quot; Conditional branching Sometimes it becomes necessary to expand a template in one of two directions based on a flag. Using the ~? function, the value will be treated as a boolean and if true will expand one branch, otherwise an alternate branch is expanded. The alternate branch is optional. Conditional branches are terminated with the ~; function, but specifying an alternate branch can be done with the ~: function. 6 &gt; F = fun ( N ) -&gt; dactyl : render ( &quot;You have ~any~? ~n ~;~:no~; eggs.&quot; , [ { any , N &gt; 0 } , { n , N } ] ) end . # Fun &lt; erl_eval . 6.80247286 &gt; 7 &gt; F ( 6 ) . &quot;You have 6 eggs.&quot; 8 &gt; F ( 0 ) . &quot;You have no eggs.&quot; Lists of parameters If you have a list of things you would like to expand multiple times (once per element in the list), then you can use the ~[ function to expand a list of parameter lists. 9 &gt; dactyl : render ( &quot;Hi, ~p eople~[ ~n ame~;~].&quot; , [ { people , [ [ { name , &quot;Jeff&quot; } ] , [ { name , &quot;Tony&quot; } ] , [ { name , &quot;Mark&quot; } ] ] } ] ) . &quot;Hi, Jeff Tony Mark.&quot; The people parameter pointed to a list, where each list was a new proplist that is then used for the inner template. Custom formatting Dactyl allows you to use the full power of Erlang's io_lib:format/2 function as well by utilizing the ~{ expansion function. Once inside the ~{ function, you can then treat the template as the Format parameter to io_lib:format/2 and the parameter value is used as the argument list to io_lib:format/2 . 10 &gt; dactyl : render ( &quot;Value = ~foo~{ ~p ~}&quot; , [ { foo , [ { a , b , c } ] } ] ) . &quot;Value = {a,b,c}&quot;","ref":"readme.html#understanding-parameter-formatting","title":"Overview - Understanding parameter formatting","type":"extras"},{"doc":"Finally, not all parameters can be calculated ahead of time, and sometimes you will not want (or can't) use atoms as the keys to your template parameters. In such cases, you can use the dactyl:render/3 and dactyl:render_file/3 functions, which allow you to pass a module name as well. During expansion of the template, the module will first be checked to see if it exports the parameter name as a function with an arity of 1. If it does, the function will be called with the proplist of parameters and its return value used. If the function is not exported, then the parameter list will be used as usual. - module ( test ) . - compile ( [ export_all , nowarn_export_all ] ) . name_begins_with_vowel ( Args ) -&gt; [ C | _ ] = proplists : get_value ( name , Args ) , lists : member ( C , &quot;aeiouAEIOU&quot; ) . generate ( Template , Name ) -&gt; dactyl : render ( ? MODULE , Template , [ { name , Name } ] ) . The above module can then be used with a template like so: 1 &gt; test : generate ( &quot; ~n ame_begins_with_vowel~? ~n ame~;~:invalid~;&quot; , &quot;Bob&quot; ) . &quot;invalid&quot; 2 &gt; test : generate ( &quot; ~n ame_begins_with_vowel~? ~n ame~;~:invalid~;&quot; , &quot;Andy&quot; ) . &quot;Andy&quot;","ref":"readme.html#exploding-parameters-with-functions","title":"Overview - Exploding parameters with functions","type":"extras"},{"doc":"All notable changes to this project will be documented in this file. The format is based on Keep a Changelog and this project adheres to Semantic Versioning .","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Added rebar3_hank [Paulo Oliveira] rebar3_ex_doc + make ex_doc-dry [Paulo Oliveira] Changed CI container approach to setup-beam with cache [Paulo Oliveira] edoc -based doc. to ex_doc -based doc. [Paulo Oliveira]","ref":"changelog.html#1-0-1-2022-04-19","title":"Changelog - [1.0.1] - 2022-04-19","type":"extras"},{"doc":"Added first version (README.md has notes on capabilities)","ref":"changelog.html#1-0-0-2021-03-04","title":"Changelog - [1.0.0] - 2021-03-04","type":"extras"}]